@@ -1,31 +1,62 @@
-use tokio::io::AsyncWriteExt;
+use futures::future::LocalBoxFuture;
+use futures::FutureExt;
+use std::future::Future;
+
 use ya_runtime_api::server::proto::{output::Type, request::RunProcess, Output};
 
 use crate::cli::{Command, CommandCli};
+use crate::common::write_output;
+use crate::context::Context;
 use crate::env::{DefaultEnv, Env};
-use crate::runtime::{Context, Runtime, RuntimeMode};
+use crate::runtime::{Runtime, RuntimeDef, RuntimeMode};
 use crate::server::Server;
 
-/// Starts the runtime
-pub async fn run<R: Runtime + 'static>() -> anyhow::Result<()> {
-    run_with::<R, _>(DefaultEnv::default()).await
+/// Starts the runtime within a new `tokio::task::LocalSet`
+#[inline]
+pub async fn run<R>() -> anyhow::Result<()>
+where
+    R: Runtime + Default + 'static,
+{
+    run_with::<R, _>(DefaultEnv::<<R as RuntimeDef>::Cli>::default()).await
 }
 
-/// Starts the runtime using a custom environment configuration provider
-pub async fn run_with<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow::Result<()> {
-    tokio::task::spawn_blocking(move || {
-        let handle = tokio::runtime::Handle::current();
-        handle.block_on(async {
-            let set = tokio::task::LocalSet::new();
-            set.run_until(inner::<R, E>(env)).await
-        })
-    })
-    .await?
+/// Starts the runtime within a new `tokio::task::LocalSet`,
+/// using a custom environment configuration provider
+pub async fn run_with<R, E>(env: E) -> anyhow::Result<()>
+where
+    R: Runtime + Default + 'static,
+    E: Env<<R as RuntimeDef>::Cli> + Send + 'static,
+{
+    build(env, move |_| async move { Ok(R::default()) }).await
 }
 
-async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow::Result<()> {
-    let mut runtime = R::default();
+/// Creates a new runtime execution future within a new `tokio::task::LocalSet`,
+/// using a custom environment configuration provider and a runtime factory
+pub fn build<R, E, F, Fut>(env: E, factory: F) -> LocalBoxFuture<'static, anyhow::Result<()>>
+where
+    R: Runtime + 'static,
+    E: Env<<R as RuntimeDef>::Cli> + Send + 'static,
+    F: FnOnce(&mut Context<R>) -> Fut + 'static,
+    Fut: Future<Output = anyhow::Result<R>> + 'static,
+{
+    async move {
+        let set = tokio::task::LocalSet::new();
+        set.run_until(inner::<R, E, _>(env, |ctx| {
+            async move { factory(ctx).await }.boxed_local()
+        }))
+        .await
+    }
+    .boxed_local()
+}
+
+async fn inner<R, E, F>(env: E, factory: F) -> anyhow::Result<()>
+where
+    R: Runtime + 'static,
+    E: Env<<R as RuntimeDef>::Cli> + Send + 'static,
+    F: FnOnce(&mut Context<R>) -> LocalBoxFuture<anyhow::Result<R>>,
+{
     let mut ctx = Context::<R>::try_with(env)?;
+    let mut runtime = factory(&mut ctx).await?;
 
     match ctx.cli.command() {
         Command::Deploy { .. } => {
@@ -42,12 +73,12 @@ async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow:
                     })
                 }
             };
-            output(deployment).await?;
+            write_output(deployment).await?;
         }
         Command::Start { .. } => match R::MODE {
             RuntimeMode::Command => {
                 if let Some(started) = runtime.start(&mut ctx).await? {
-                    output(started).await?;
+                    write_output(started).await?;
                 }
             }
             RuntimeMode::Server => {
@@ -58,13 +89,7 @@ async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow:
                     };
 
                     if let Some(out) = start.await.expect("Failed to start the runtime") {
-                        crate::runtime::RunCommandContext {
-                            id: ctx.next_pid(),
-                            emitter: ctx.emitter.clone(),
-                            control: Default::default(),
-                        }
-                        .stdout(out.to_string())
-                        .await;
+                        ctx.next_run_ctx().stdout(out.to_string()).await;
                     }
 
                     Server::new(runtime, ctx)
@@ -88,7 +113,7 @@ async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow:
                 work_dir: ctx
                     .cli
                     .workdir()
-                    .unwrap_or_else(Default::default)
+                    .unwrap_or_default()
                     .to_string_lossy()
                     .to_string(),
                 stdout: capture.clone(),
@@ -100,12 +125,12 @@ async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow:
                 .await?;
 
             if let RuntimeMode::Server = R::MODE {
-                output(serde_json::json!(pid)).await?;
+                write_output(serde_json::json!(pid)).await?;
             }
         }
         Command::OfferTemplate { .. } => {
             if let Some(template) = runtime.offer(&mut ctx).await? {
-                output(template).await?;
+                write_output(template).await?;
             }
         }
         Command::Test { .. } => runtime.test(&mut ctx).await?,
@@ -113,11 +138,3 @@ async fn inner<R: Runtime + 'static, E: Env + Send + 'static>(env: E) -> anyhow:
 
     Ok(())
 }
-
-async fn output(json: serde_json::Value) -> anyhow::Result<()> {
-    let string = json.to_string();
-    let mut stdout = tokio::io::stdout();
-    stdout.write_all(string.as_bytes()).await?;
-    stdout.flush().await?;
-    Ok(())
-}
